using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace Mycos.Controls.WinControls
{
	/// <summary>
	/// A cylindrical track bar filled with a gradient
	/// * No warranty or gaurantee expressed or implied *
	/// </summary>
	[DefaultEvent("ValueChanged")]
	public class CylinderTrackBar : Mycos.Controls.WinControls.RoundControl
	{
		/// <summary> 
		/// Fields
		/// </summary>
		private System.ComponentModel.Container components = null;

		private MouseState		mouseState;
		private int				alpha;
		private float			max;
		private float			min;
		private float			_value;
		private Color			barColor;
		private Color			outLineColor;
		private bool			showValue;
		private Point			oldPosition;
		private Point			mousePosition;

		//
		// Events
		//
		/// <summary>
		/// Occurs when value was changed.
		/// </summary>
		public event EventHandler ValueChanged;

		/// <summary>
		/// Initializes a new instance of the CloseButton class.
		/// </summary>
		public CylinderTrackBar() : base()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			//
			// Initialize Value
			//
			this.mouseState = MouseState.Normal;
			this.alpha = 255;
			this.barColor = Color.Gold;
			this.max = 100;
			this._value = 1;
			this.min = 0;
			this.outLineColor = Color.Gray;
			this.ForeColor = Color.LightGray;
			this.showValue = true;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// CylinderTrackBar
			// 
			this.ForeColor = System.Drawing.Color.Black;
			this.Name = "CylinderTrackBar";
			this.Size = new System.Drawing.Size(56, 168);

		}
		#endregion

		#region Public Properties

		/// <summary>
		/// Inner color - will appear as a gradient
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(typeof(Color), "Gold")]
		public Color BarColor
		{
			get
			{
				return this.barColor;
			}
			set
			{
				if ( this.barColor != value )
				{
					this.barColor = value;
					this.Invalidate();
				}
			}
		}

		/// <summary>
		/// Outline color
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(typeof(Color), "Gray")]
		public Color OutLineColor
		{
			get
			{
				return this.outLineColor;
			}
			set
			{
				if ( this.outLineColor != value )
				{
					this.outLineColor = value;
					this.Invalidate();
				}
			}
		}

		/// <summary>
		/// Determines the maximum value with the default set to 100
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(100f)]
		public float Maximum
		{
			get
			{
				return this.max;
			}
			set
			{
				if ( this.max != value )
				{
					this.max = value;
				}
			}
		}

		/// <summary>
		/// Determines the minimum value (defaults to 0)
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(0f)]
		public float Minimum
		{
			get
			{
				return this.min;
			}
			set
			{
				if ( this.min != value )
				{
					this.min = value;
				}
			}
		}

		/// <summary>
		/// Get or set the current Value
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(1f)]
		public float Value
		{
			get
			{
				return this._value ;
			}
			set
			{
				if ( this._value != value )
				{
					this._value = value;
					this.OnValueChanged(EventArgs.Empty);
				}
			}
		}

		/// <summary>
		/// Customize the transparency
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(255)]
		public int Alpha
		{
			get
			{
				return this.alpha;
			}
			set
			{
				if ( this.alpha != value )
				{
					this.alpha = value;
					this.Invalidate();
				}
			}
		}

		/// <summary>
		/// Toggle the display of the value when resizing
		/// </summary>
		[Category("Mycos TrackBar"),
		DefaultValue(false)]
		public bool ShowValue
		{
			get
			{
				return this.showValue;
			}
			set
			{
				if ( this.showValue != value )
				{
					this.showValue = value;
					this.Invalidate();
				}
			}
		}
		#endregion

		#region Paint Code.

		/// <summary>
		/// Overridden OnPaint for custom painting.
		/// </summary>
		/// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
		protected override void OnPaint(PaintEventArgs pe)
		{
			base.OnPaint(pe);
			Graphics g = pe.Graphics;
			g.SmoothingMode = SmoothingMode.AntiAlias;

			if ( this.mouseState == MouseState.Normal )
				this.DrawBar(g, this.ForeColor);
			else if ( this.mouseState == MouseState.Hot )
				this.DrawBar(g, Color.WhiteSmoke);
			else if ( this.mouseState == MouseState.HOver )
				this.DrawBar(g, Color.LightGray);

			if ( this.mouseState == MouseState.Hot )
				this.Cursor = System.Windows.Forms.Cursors.HSplit;
			else
				this.Cursor = System.Windows.Forms.Cursors.Default;
		}//end Paint

		/// <summary>
		/// Do the actual drawing here
		/// </summary>
		/// <param name="g"></param>
		/// <param name="foreColor"></param>
		private void DrawBar(Graphics g, Color foreColor)
		{
			bool outLine = this.outLineColor != Color.Transparent;
			Rectangle bound = this.ClientRectangle;

			bound.Inflate(-20, -20);
			Mycos.Glib.DrawingUtils.FillCylinder(g, bound, new Size(4, 10), Color.FromArgb(this.Alpha, foreColor), this.OutLineColor, outLine);

			if ( this.Value > this.Minimum && this.Value <= this.Maximum )
			{
				float barValue = bound.Height * (( this.Value - this.Minimum ) / ( this.Maximum - this.Minimum));
				RectangleF valueBound = RectangleF.FromLTRB(bound.Left, bound.Bottom - barValue, bound.Right, bound.Bottom);

			
				Mycos.Glib.DrawingUtils.FillCylinder(g, valueBound, new Size(4, 10), Color.FromArgb(this.Alpha, this.BarColor), this.OutLineColor, outLine);

				if ( this.showValue && valueBound.Height > 20 )
				{
					g.SmoothingMode = SmoothingMode.AntiAlias;
					StringFormat format = new StringFormat();
					format.Alignment = StringAlignment.Center;
					format.LineAlignment = StringAlignment.Center;
					g.DrawString(this._value.ToString("F2"), this.Font, Brushes.Black, valueBound, format);
					format.Dispose();
				}//end if Show Value
			}//end in Length
		}
		#endregion

		#region EventHandler.

		/// <summary>
		/// Handle mouse down
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);
			this.mouseState = MouseState.Hot;

			this.Capture = true;

			this.oldPosition = this.mousePosition = new Point(e.X, e.Y);

		}

		/// <summary>
		/// Handle mouse up
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseUp(MouseEventArgs e)
		{
			this.mouseState = MouseState.Normal;

			this.Capture = false;

			base.OnMouseUp(e);
			this.Invalidate();
		}

		/// <summary>
		/// Handle mouse enter
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseEnter(EventArgs e)
		{
			this.mouseState = MouseState.HOver;

			base.OnMouseEnter(e);
			this.Invalidate();
		}

		/// <summary>
		/// Handle mouse move
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseMove(MouseEventArgs e)
		{
			this.mousePosition = new Point(e.X, e.Y);
			RectangleF bounds = this.ClientRectangle;

			bounds.Inflate(-20, -20);

			if(this.Capture)
			{
				if(bounds.Contains(mousePosition))
				{
					this.mouseState = MouseState.Hot;
					this.Value = this.Minimum + (( bounds.Bottom - e.Y  ) / bounds.Height ) * ( this.Maximum - this.Minimum );
				}
				else
					this.mouseState = MouseState.HOver;
			}
			this.Invalidate();
		}

		/// <summary>
		/// Handle mouse leave
		/// </summary>
		/// <param name="e"></param>
		protected override void OnMouseLeave(EventArgs e)
		{
			this.mouseState = MouseState.Normal;
			base.OnMouseLeave(e);
			this.Invalidate();
		}

		/// <summary>
		/// On value changed
		/// </summary>
		/// <param name="e"></param>
		private void OnValueChanged(EventArgs e)
		{
			if ( this.ValueChanged != null )
				this.ValueChanged(this, e);

			this.Invalidate();
		}
		#endregion

	}
}
